﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;

namespace Magnetometer_Example {
	public partial class Form1 : Form {
		CommandLineOpen open;
        formCleaner cleaner;
        Magnetometer mag = null;
		private ErrorEventBox errorBox;

		double ambientMagneticField = 0.57142; //Calgary

		public Form1() {
			open = new CommandLineOpen(this);
            cleaner = new formCleaner(this);
            InitializeComponent();
		}

		public Form1(String[] commandLine) {
			open = new CommandLineOpen(this);
            cleaner = new formCleaner(this);
            open.commandLine = commandLine;
			InitializeComponent();
		}

		private void Form1_Load(object sender, EventArgs e) {
			Phidget.InvokeEventCallbacks = true;
			errorBox = new ErrorEventBox(this);
			mag = new Magnetometer();

			initMagFieldGraph();

			mag.Attach += magnetometer_attach;
			mag.Detach += magnetometer_detach;
			mag.Error += magnetometer_error;
			mag.MagneticFieldChange += fieldStrength_change;

			open.openCmdLine(mag);

		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {
			mag.Close();
		}


		void magnetometer_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);

			settingsBox.Visible = true;
			outputBox.Visible = true;
			magFieldGraphBox.Visible = true;

			Magnetometer attachedDevice = (Magnetometer)sender;

			try {
				attachedDevice.ResetCorrectionParameters();

				dataIntervalTrk.Unit = "ms";
				dataIntervalTrk.minTxt = attachedDevice.MinDataInterval.ToString();
				dataIntervalTrk.Minimum = attachedDevice.MinDataInterval;
				dataIntervalTrk.maxTxt = attachedDevice.MaxDataInterval.ToString();
				dataIntervalTrk.Maximum = attachedDevice.MaxDataInterval;
				dataIntervalTrk.trkText = attachedDevice.DataInterval.ToString();
				dataIntervalTrk.Value = attachedDevice.DataInterval;

				changeTriggerTrk.Unit = "G";
				changeTriggerTrk.minTxt = attachedDevice.MinMagneticFieldChangeTrigger.ToString();
				changeTriggerTrk.Minimum = attachedDevice.MinMagneticFieldChangeTrigger;
				changeTriggerTrk.Maximum = attachedDevice.MaxMagneticFieldChangeTrigger;
				changeTriggerTrk.maxTxt = attachedDevice.MaxMagneticFieldChangeTrigger.ToString();
				changeTriggerTrk.trkText = attachedDevice.MagneticFieldChangeTrigger.ToString();
				changeTriggerTrk.Value = attachedDevice.MagneticFieldChangeTrigger;

			} catch (PhidgetException ex) {
				errorBox.addMessage("Error initializing device: " + ex.Message);
			}
		}

		void magnetometer_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			phidgetInfoBox1.Clear();
            cleaner.clean();

            settingsBox.Visible = false;
			outputBox.Visible = false;
			magFieldGraphBox.Visible = false;
			calibrationStatusTxt.Visible = false;
		}

		void magnetometer_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);
		}

		void fieldStrength_change(object sender, Phidget22.Events.MagnetometerMagneticFieldChangeEventArgs e) {
			xaxistxt.Text = e.MagneticField[0].ToString() + " G";
			yaxistxt.Text = e.MagneticField[1].ToString() + " G";
			zaxistxt.Text = e.MagneticField[2].ToString() + " G";
			timestamptxt.Text = e.Timestamp.ToString() + " ms";

			try {
				drawMagFieldGraph();
			} catch { }
		}

		private void dataIntervalTrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					mag.DataInterval = (int)dataIntervalTrk.Value;
				} catch (Exception ex) {
					errorBox.addMessage("Error setting data interval: " + ex.Message);
				}
			}
		}

		private void dataIntervalTrk_LogMouseUp(object sender, MouseEventArgs e) {
			try {
				mag.DataInterval = (int)dataIntervalTrk.Value;
			} catch (Exception ex) {
				errorBox.addMessage("Error setting data interval: " + ex.Message);
			}
		}



		private void changeTriggertrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					mag.MagneticFieldChangeTrigger = changeTriggerTrk.Value;
				} catch (Exception ex) {
					errorBox.addMessage("Error setting change trigger: " + ex.Message);
				}
			}
		}


		private void changeTriggerTrk_LogMouseUp(object sender, MouseEventArgs e) {
			try {
				mag.MagneticFieldChangeTrigger = changeTriggerTrk.Value;
			} catch (Exception ex) {
				errorBox.addMessage("Error setting change trigger: " + ex.Message);
			}
		}



		private void setCompassParamsButton_Click(object sender, EventArgs e) {


			String calibrationString = "0.41595, 0.02233, 0.15236, -0.22662, 2.36152, 2.27444, 2.57652, -0.03768, -0.01708, -0.03607, -0.00707, -0.01822, -0.00787"; //test string based on Calgary, AB and a prototype magnetometer

			ParameterInputBox tib = new ParameterInputBox(calibrationString);
			DialogResult result;
			result = tib.ShowDialog();

			if (result == DialogResult.OK) {
				if (tib.parameters.Length > 0)
					calibrationString = tib.parameters;
				else {
					calibrationStatusTxt.Text = "Un-Calibrated";
					calibrationStatusTxt.ForeColor = Color.Red;
					calibrationStatusTxt.Visible = true;
				}
				string[] paramVals = calibrationString.Split(new char[] { ',' });
				if (paramVals.Length == 13) {
					try {
						mag.SetCorrectionParameters(
							double.Parse(paramVals[0]),
							double.Parse(paramVals[1]),
							double.Parse(paramVals[2]),
							double.Parse(paramVals[3]),
							double.Parse(paramVals[4]),
							double.Parse(paramVals[5]),
							double.Parse(paramVals[6]),
							double.Parse(paramVals[7]),
							double.Parse(paramVals[8]),
							double.Parse(paramVals[9]),
							double.Parse(paramVals[10]),
							double.Parse(paramVals[11]),
							double.Parse(paramVals[12])
						);
						ambientMagneticField = double.Parse(paramVals[0]);
						calibrationStatusTxt.Text = "Calibrated";
						calibrationStatusTxt.ForeColor = Color.Green;
						calibrationStatusTxt.Visible = true;
					} catch (PhidgetException ex) { errorBox.addMessage("Error calibrating: " + ex.Message); }
				}

			}
		}

		#region graph
		private BufferedGraphicsContext magFieldGraphicsContext;
		private BufferedGraphics magFieldGraphicsBuffer;
		private Bitmap magFieldDrawingSurface;

		private Rectangle magFieldCircleRectangle, magFieldBoundsRectangle;
		private Pen magFieldXYAxisPen, magFieldCirclePen;
		private float magFieldCircleDiameter, magFieldCircleRadius;



		private void initMagFieldGraph() {

			//initialize our graphical representation of the compasseration data
			magFieldDrawingSurface = new Bitmap(magFieldView.ClientRectangle.Width, magFieldView.ClientRectangle.Height, System.Drawing.Imaging.PixelFormat.Format24bppRgb);

			magFieldBoundsRectangle = new Rectangle(0, 0, magFieldDrawingSurface.Width, magFieldDrawingSurface.Height);
			magFieldBoundsRectangle.Inflate(-1, -1);

			float xCenter = (float)(magFieldBoundsRectangle.Width / 2.0);
			float yCenter = (float)(magFieldBoundsRectangle.Height / 2.0);

			magFieldXYAxisPen = new Pen(Color.Black, 2);
			magFieldCirclePen = new Pen(Color.DarkBlue, 2);

			magFieldCircleDiameter = (float)(magFieldDrawingSurface.Width / 1.1);
			magFieldCircleRadius = magFieldCircleDiameter / 2;

			magFieldCircleRectangle = new Rectangle((int)Math.Round(xCenter - magFieldCircleRadius), (int)Math.Round(yCenter - magFieldCircleRadius), (int)Math.Round(magFieldCircleDiameter), (int)Math.Round(magFieldCircleDiameter));
		}

		private void drawMagFieldGraph() {
			magFieldGraphicsContext = new BufferedGraphicsContext();
			magFieldGraphicsBuffer = magFieldGraphicsContext.Allocate(Graphics.FromImage(magFieldDrawingSurface), magFieldBoundsRectangle);
			magFieldGraphicsBuffer.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
			magFieldGraphicsBuffer.Graphics.Clear(SystemColors.Control);

			float xCenter = (float)(magFieldBoundsRectangle.Width / 2.0);
			float yCenter = (float)(magFieldBoundsRectangle.Height / 2.0);

			float xOld = xCenter + (float)(mag.MagneticField[0]) * magFieldCircleRadius * (float)(1 / ambientMagneticField);
			float yOld = yCenter + (float)(mag.MagneticField[1]) * magFieldCircleRadius * (float)(1 / ambientMagneticField);

			magFieldGraphicsBuffer.Graphics.DrawLine(magFieldXYAxisPen, xCenter, yCenter, xOld, yOld);
			magFieldGraphicsBuffer.Graphics.DrawEllipse(magFieldCirclePen, magFieldCircleRectangle);

			if (mag.MagneticField.Length == 3) {
				double zOut = (mag.MagneticField[2]) * (float)(1 / ambientMagneticField);
				if (zOut > 0) {
					magFieldGraphicsBuffer.Graphics.DrawEllipse(new Pen(Color.Red, 2),
						new Rectangle((int)xCenter - (int)(magFieldCircleRadius * zOut), (int)yCenter - (int)(magFieldCircleRadius * zOut),
						(int)(magFieldCircleDiameter * zOut), (int)(magFieldCircleDiameter * zOut)));
				} else {
					magFieldGraphicsBuffer.Graphics.DrawEllipse(new Pen(Color.Green, 2),
						new Rectangle((int)xCenter - (int)(magFieldCircleRadius * -zOut), (int)yCenter - (int)(magFieldCircleRadius * -zOut),
						(int)(magFieldCircleDiameter * -zOut), (int)(magFieldCircleDiameter * -zOut)));
				}
			}

			magFieldGraphicsBuffer.Render(magFieldView.CreateGraphics());

			magFieldGraphicsBuffer.Dispose();

			magFieldGraphicsContext.Dispose();
		}
		#endregion


	}
}
